from datetime import datetime
from enum import Enum, auto


class AutoName(Enum):
    def _generate_next_value_(name, start, count, last_values):
        return name.lower()


class Condition(AutoName):
    fog = auto()
    hazemoon = auto()
    hazesun = auto()
    lightning = auto()
    mist = auto()
    moon = auto()
    mooncloud = auto()
    rain = auto()
    sun = auto()
    suncloud = auto()
    storm = auto()


class Weather:
    """
    A class encapsulating raw weather data.

    :param reference_time: Timestamp of weather measurement
    :type reference_time: `datetime.datetime`
    :param condition: Single-word weather condition
    :type condition: `meteoclimatic.Condition` or `str`
    :param temp_current: Current temperature in Celsius
    :type temp_current: `float`
    :param temp_max: Maximum temperature in Celsius for the past 24 hours
    :type temp_max: `float`
    :param temp_min: Minimum temperature in Celsius for the past 24 hours
    :type temp_min: `float`
    :param humidity_current: Current humidity in percentage points
    :type humidity_current: `float`
    :param humidity_max: Maximum humidity in percentage points for the past 24 hours
    :type humidity_max: `float`
    :param humidity_min: Minimum humidity in percentage points for the past 24 hours
    :type humidity_min: `float`
    :param pressure_current: Current atmospheric pressure in hPa units
    :type pressure_current: `float`
    :param pressure_max: Maximum atmospheric pressure in hPa units for the past 24 hours
    :type pressure_max: `float`
    :param pressure_min: Minimum atmospheric pressure in hPa units for the past 24 hours
    :type pressure_min: `float`
    :param wind_current: Current wind speed in km/h units
    :type wind_current: `float`
    :param wind_max: Maximum wind speed in km/h units for the past 24 hours
    :type wind_max: `float`
    :param wind_bearing: Wind bearing in degree units
    :type wind_bearing: `float`
    :param rain: Precipitation in mm units for the past 24 hours
    :type rain: `float`
    :returns: a *Weather* instance
    :raises: *ValueError* when invalid values are provided for non-negative or ranged
        quantities
    """

    def __init__(self, reference_time: datetime, condition: Condition,
                 temp_current: float, temp_max: float, temp_min: float,
                 humidity_current: float, humidity_max: float, humidity_min: float,
                 pressure_current: float, pressure_max: float, pressure_min: float,
                 wind_current: float, wind_max: float, wind_bearing: float,
                 rain: float):
        """Initialize the class."""
        if not isinstance(reference_time, datetime):
            raise ValueError(
                "reference_time is not an instance of datetime.datetime")
        self.reference_time = reference_time

        self.condition = condition

        self.temp_current = temp_current
        self.temp_max = temp_max
        self.temp_min = temp_min

        for humidity in [humidity_current, humidity_max, humidity_min]:
            if humidity is not None and (humidity < 0.0 or humidity > 100.0):
                raise ValueError("humidity must be between 0 and 100")
        self.humidity_current = humidity_current
        self.humidity_max = humidity_max
        self.humidity_min = humidity_min

        self.pressure_current = pressure_current
        self.pressure_max = pressure_max
        self.pressure_min = pressure_min

        for wind in [wind_current, wind_max]:
            if wind is not None and wind < 0.0:
                raise ValueError("wind must be greatear than 0")
        self.wind_current = wind_current
        self.wind_max = wind_max

        if wind_bearing is not None and (wind_bearing < 0.0 or wind_bearing > 360.0):
            raise ValueError("wind bearing must be between 0 and 360")
        self.wind_bearing = wind_bearing

        if rain is not None and rain < 0.0:
            raise ValueError("rain must be greatear than 0")
        self.rain = rain

    def __eq__(self, other):
        if not isinstance(other, Weather):
            return NotImplemented
        prop_names = list(self.__dict__)
        for prop in prop_names:
            if self.__dict__[prop] != other.__dict__[prop]:
                return False
        return True

    def __repr__(self):
        return "%s(%r)" % (self.__class__, self.__dict__)



class Station:
    """
    A class representing a Meteoclimatic station.

    :param name: Name of the station
    :type name: `str`
    :param code: Meteoclimatic code of the station
    :type code: `str`
    :param url: URL of the station
    :type url: `str`
    :returns: a *Station* instance
    :raises: *ValueError* when invalid empty or null values are provided
    """

    def __init__(self, name: str, code: str, url: str):
        """Initialize the class."""
        if name is None or len(name) == 0:
            raise ValueError("Station name cannot be empty")
        self.name = name

        if code is None or len(code) == 0:
            raise ValueError("Station code cannot be empty")
        self.code = code
        self.url = url

    def __eq__(self, other):
        if not isinstance(other, Station):
            return NotImplemented
        prop_names = list(self.__dict__)
        for prop in prop_names:
            if self.__dict__[prop] != other.__dict__[prop]:
                return False
        return True

    def __repr__(self):
        return "%s(%r)" % (self.__class__, self.__dict__)



import logging
from datetime import datetime
from meteoclimatic import Station, Weather, Condition
from meteoclimatic.feed import FeedItemHelper


class Observation:
    """
    A class representing the weather which is currently being observed from a
    certain station. The station is represented by the encapsulated *Station*
    object while the observed weather data are held by the encapsulated *Weather*
    object.

    :param reception_time: Timestamp telling when the weather obervation has
        been received from the station
    :type reception_time: `datetime.datetime`
    :param station: the *Station* relative to this observation
    :type station: *Station*
    :param weather: the *Weather* relative to this observation
    :type weather: *Weather*
    :returns: an *Observation* instance
    :raises: *ValueError* when negative values are provided as reception time
    """

    _feed_datetime_format = "%a, %d %b %Y %H:%M:%S %z"

    def __init__(self, reception_time: datetime, station: Station, weather: Weather):
        """Initialize the class."""
        if not isinstance(reception_time, datetime):
            raise ValueError(
                "reception_time is not an instance of datetime.datetime")
        self.reception_time = reception_time

        if not isinstance(station, Station):
            raise ValueError(
                "station is not an instance of meteoclimatic.Station")
        self.station = station

        if not isinstance(weather, Weather):
            raise ValueError(
                "weather is not an instance of meteoclimatic.Weather")
        self.weather = weather

    @classmethod
    def from_feed_item(cls, feed_item):
        """
        Parses an *Observation* instance out of an RSS feed item.
        :param feed_item: the input RSS feed item
        :type feed_item: `bs4.element.Tag`
        :returns: an *Observation* instance
        :raises: *ValueError* if it is not possible to parse the data
        """
        helper = FeedItemHelper(feed_item)

        station_name = feed_item.title.text
        station_code = helper.get_text("station_code")
        station_url = feed_item.link.text
        station = Station(station_name, station_code, station_url)

        reception_time = datetime.strptime(
            feed_item.pubDate.text, cls._feed_datetime_format)

        condition_str = helper.get_text("condition")
        try:
            condition = Condition(condition_str)
        except ValueError:
            logging.info(
                "Unrecognized condidition '%s', using literal value instead of meteoclimatic.Condition" % (condition_str, ))
            condition = condition_str
        temp_current = helper.get_float("temp_current")
        temp_max = helper.get_float("temp_max")
        temp_min = helper.get_float("temp_min")
        humidity_current = helper.get_float("humidity_current")
        humidity_max = helper.get_float("humidity_max")
        humidity_min = helper.get_float("humidity_min")
        pressure_current = helper.get_float("pressure_current")
        pressure_max = helper.get_float("pressure_max")
        pressure_min = helper.get_float("pressure_min")
        wind_current = helper.get_float("wind_current")
        wind_max = helper.get_float("wind_max")
        wind_bearing = helper.get_float("wind_bearing")
        rain = helper.get_float("rain")
        wind_max = helper.get_float("wind_max")

        weather = Weather(reception_time, condition,
                          temp_current, temp_max, temp_min,
                          humidity_current, humidity_max, humidity_min,
                          pressure_current, pressure_max, pressure_min,
                          wind_current, wind_max, wind_bearing,
                          rain)

        return cls(reception_time, station, weather)

    def __eq__(self, other):
        if not isinstance(other, Observation):
            return NotImplemented
        prop_names = list(self.__dict__)
        for prop in prop_names:
            if self.__dict__[prop] != other.__dict__[prop]:
                return False
        return True

    def __repr__(self):
        return "%s(%r)" % (self.__class__, self.__dict__)



from setuptools import setup
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pymeteoclimatic',
    version='0.1.0',
    description='A Python wrapper around the Meteoclimatic service',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Adrián Moreno',
    author_email='adrian@morenomartinez.com',
    url='https://github.com/adrianmo/pymeteoclimatic',
    packages=['meteoclimatic', ],
    install_requires=['lxml>=4.5',
                      'beautifulsoup4>=4.9'
                      ],
    python_requires='>=3.8',
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries"],
    keywords='meteoclimatic client library api weather',
    license='MIT',
)



import re


class FeedItemHelper(object):
    """Helper class to get content from a Meteoclimatic RSS feed item."""

    _regex_pattern = r"\[\[\<(?P<station_code>\w+);\((?P<temp_current>-?[0-9,]+);(?P<temp_max>-?[0-9,]+);(?P<temp_min>-?[0-9,]+);(?P<condition>\w*)\);\((?P<humidity_current>-?[0-9,]*);(?P<humidity_max>-?[0-9,]*);(?P<humidity_min>-?[0-9,]*)\);\((?P<pressure_current>-?[0-9,]*);(?P<pressure_max>-?[0-9,]*);(?P<pressure_min>-?[0-9,]*)\);\((?P<wind_current>-?[0-9,]*);(?P<wind_max>-?[0-9,]*);(?P<wind_bearing>-?[0-9,]*)\);\((?P<rain>-?[0-9,]*)\);"  # noqa: E501

    def __init__(self, feed_item):
        """Initialize the class."""
        self.match = re.search(self._regex_pattern, str(feed_item.description))
        if not self.match:
            raise ValueError("Could not parse station information")

    def get_text(self, field_name):
        """Return the value in 'field_name' from the item or None if not found."""
        try:
            value = self.match.group(field_name)
        except IndexError:
            return None
        if len(value) == 0:
            return None
        return value

    def get_float(self, field_name):
        """Return the value in 'field_name' from the item or None if not found."""
        value = self.get_text(field_name)
        if value is None:
            return None
        try:
            value = float(value.replace(",", "."))
        except ValueError:
            return None
        if value == -99.0:
            # Meteoclimatic returns -99,0 when the station does not provide the value
            return None
        return value
